import { transform } from 'lodash';
import * as _ from 'lodash';
import { ErrorType, HttpStatusCode } from 'azure-functions-ts-essentials';
import { Typegoose } from 'typegoose';

class BaseDocument extends Typegoose {
}

function connect(instance, connStr, timeout = 250, retry = 0) {
    return new Promise((resolve, reject) => instance.connect(connStr, { useMongoClient: true }, (err) => err
        ? retry < 100
            ? setTimeout(() => resolve(connect(instance, connStr)), timeout)
            : reject(err)
        : resolve()));
}
function parseFields(rawFields) {
    if (!rawFields)
        return {};
    return rawFields.split(',')
        .map(cur => String(cur).trim())
        .reduce((acc, cur) => {
        acc[cur] = 1;
        return acc;
    }, {});
}
const parseQueryValue = (queryValue) => {
    queryValue = decodeURIComponent(queryValue).trim();
    if (queryValue.toLowerCase() === 'null')
        return null;
    if (queryValue.toLowerCase() === 'undefined')
        return undefined;
    if (queryValue.toLowerCase() === 'true')
        return true;
    else if (queryValue.toLowerCase() === 'false')
        return false;
    else if (queryValue === '0')
        return 0;
    else if (Number(queryValue) !== 0 && !isNaN(Number(queryValue)))
        return Number(queryValue);
    else
        return { $regex: queryValue, $options: 'i' };
};
function parseQuery(rawQuery) {
    const res = {};
    if (!rawQuery)
        return res;
    rawQuery.split(',').forEach(segment => {
        if (!segment)
            return {};
        const parts = segment.match(/([^,]+)::([^,]+|)?/);
        if (!(parts && parts.length > 0))
            return {};
        const path = parts[1].match(/([^.]+)/g);
        let current = res;
        path.forEach((m, i) => {
            if (!current[m])
                current[m] = {};
            if (i === path.length - 1)
                current[m] = (!parts[2])
                    ? ''
                    : parseQueryValue(parts[2]);
            else
                current = current[m];
        });
    });
    return res;
}
const appendObject = (obj, path) => {
    const keys = path.split(':');
    const lastKey = keys.pop();
    const lastObj = keys.reduce((acc, cur) => acc[cur] = acc[cur] || {}, obj);
    lastObj[lastKey] = 'path';
    return obj;
};
const toPopulation = (obj) => {
    return transform(obj, (res, value, key) => {
        if (typeof (value) === 'object')
            res.push({
                path: key,
                populate: toPopulation(value)
            });
        else
            res.push({ path: key });
    }, []);
};
function parsePopulation(rawPopulation) {
    if (!rawPopulation)
        return '';
    let obj = {};
    for (const item of rawPopulation.split(','))
        obj = appendObject(obj, item);
    return toPopulation(obj);
}
function parseSort(rawSort) {
    return rawSort.replace(/,/g, ' ');
}
const getErrorResponse = (err) => {
    let status = HttpStatusCode.InternalServerError;
    let type = '';
    if (err.name === 'MongoError' && err.code === 11000) {
        status = HttpStatusCode.Conflict;
        type = ErrorType.AlreadyExists;
    }
    else if (err.name === 'ValidationError') {
        status = HttpStatusCode.UnprocessableEntity;
        type = ErrorType.MissingField;
    }
    return {
        status,
        body: {
            type,
            message: err.message
        }
    };
};
function clearCollection(instance, name) {
    const collection = instance.connection.collections[name];
    return collection.drop()
        .catch(() => clearCollection(instance, name));
}
class Mongooser {
    constructor(model) {
        this.model = model;
    }
    getOne(id, projection, population) {
        const query$ = this.model.findOne({ _id: id }, projection).populate(population).lean();
        return query$
            .then((doc) => {
            if (!doc)
                return Promise.resolve({
                    status: HttpStatusCode.NotFound
                });
            const data = doc;
            data._id = String(doc._id);
            return {
                status: HttpStatusCode.OK,
                body: Object.assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
            };
        })
            .catch(getErrorResponse);
    }
    getMany(criteria, projection, population, page, perPage, sort, showInactive) {
        if (!criteria.hasOwnProperty('isActive') && !showInactive)
            criteria = Object.assign({}, criteria, { isActive: true });
        const count$ = this.model
            .find(criteria, projection)
            .count();
        const query$ = this.model
            .find(criteria, projection)
            .sort(sort)
            .skip(Number(page) >= 0 && Number(perPage) > 0
            ? Number(page) * Number(perPage)
            : 0)
            .limit(Number(page) >= 0 && Number(perPage) > 0
            ? Number(perPage)
            : 0)
            .populate(population)
            .lean();
        return Promise.all([count$, query$])
            .then((res) => {
            const totalCount = res[0];
            const docs = res[1];
            const data = [];
            for (const item of docs) {
                item._id = String(item._id);
                data.push(Object.assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: HttpStatusCode.OK,
                body: {
                    data,
                    hasMore: Number(page) >= 0 && Number(perPage) > 0
                        ? totalCount > (Number(page) + 1) * Number(perPage)
                        : false,
                    totalCount
                }
            };
        })
            .catch(getErrorResponse);
    }
    insertMany(req) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        const query$ = this.model.insertMany(req.body);
        return query$.then((docs) => {
            const data = [];
            for (const item of docs) {
                item._id = String(item._id);
                data.push(Object.assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: HttpStatusCode.Created,
                body: {
                    data,
                    hasMore: false,
                    totalCount: data.length
                }
            };
        })
            .catch(getErrorResponse);
    }
    updateOne(req, id) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        const query$ = this.model.findOneAndUpdate({ _id: id }, req.body, { new: true }).lean();
        return query$.then((doc) => {
            if (!doc)
                return {
                    status: HttpStatusCode.BadRequest,
                    body: {
                        type: ErrorType.Missing
                    }
                };
            else {
                const data = doc;
                data._id = String(doc._id);
                return {
                    status: HttpStatusCode.OK,
                    body: Object.assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
                };
            }
        })
            .catch(getErrorResponse);
    }
    deactivateOne(id) {
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Missing
                }
            });
        const query$ = this.model.findOneAndUpdate({ _id: id }, { isActive: false }).lean();
        return query$.then((doc) => {
            if (!doc)
                return {
                    status: HttpStatusCode.BadRequest
                };
            else
                return {
                    status: HttpStatusCode.OK,
                    body: {
                        deactivated: true,
                        _id: String(doc._id)
                    }
                };
        })
            .catch(getErrorResponse);
    }
}

export { BaseDocument, connect, parseFields, parseQuery, parsePopulation, parseSort, clearCollection, Mongooser };
//# sourceMappingURL=azure-functions-mongooser.js.map
